@@ -1,6 +1,6 @@
 /*
     tc_client, a simple non-flash client for tinychat(.com)
-    Copyright (C) 2014-2015  alicia@ion.nu
+    Copyright (C) 2014-2016  alicia@ion.nu
     Copyright (C) 2014-2015  Jade Lea
     Copyright (C) 2015  Pamela Hiatt
 
@@ -518,6 +518,7 @@ int main(int argc, char** argv)
                  "/camup          = open an audio/video stream for broadcasting your video\n"
                  "/camdown        = close the broadcasting stream\n"
                  "/video <length> = send a <length> bytes long encoded frame, send the frame data after this line\n"
+                 "/audio <length> = send a <length> bytes long encoded frame, send the frame data after this line\n"
                  "/topic <topic>  = set the channel topic\n"
                  "/whois <nick/ID> = check a user's username\n");
           fflush(stdout);
@@ -691,10 +692,18 @@ int main(int argc, char** argv)
         }
         else if(!strncmp(buf, "/video ", 7)) // Send video data
         {
-          size_t len=strtoul(&buf[7],0,10);
+          size_t len=strtoul(&buf[7],0,0);
           char buf[len];
           fullread(0, buf, len);
-          stream_sendvideo(sock, buf, len);
+          stream_sendframe(sock, buf, len, RTMP_VIDEO);
+          continue;
+        }
+        else if(!strncmp(buf, "/audio ", 7)) // Send audio data
+        {
+          size_t len=strtoul(&buf[7],0,0);
+          char buf[len];
+          fullread(0, buf, len);
+          stream_sendframe(sock, buf, len, RTMP_AUDIO);
           continue;
         }
         else if(!strncmp(buf, "/topic ", 7))
@@ -861,6 +870,15 @@ int main(int argc, char** argv)
           printf("%s%s", (i==2?"":", "), nick);
           if(amf_comparestrings_c(&amfin->items[0].string, "registered"))
           {
+            // Tell the server how often to let us know it got our packets
+            struct rtmp setbw={
+              .type=RTMP_SERVER_BW,
+              .chunkid=2,
+              .length=4,
+              .msgid=0,
+              .buf="\x00\x00\x40\x00" // Every 0x4000 bytes
+            };
+            rtmp_send(sock, &setbw);
             char* key=getkey(id, channel);
             curl_easy_cleanup(curl); // At this point we should be done with HTTP requests
             curl=0;
@@ -872,15 +890,16 @@ int main(int argc, char** argv)
             amfstring(&amf, key);
             amfsend(&amf, sock);
             free(key);
-            if(nickname[0]) // Empty = don't set one
-            {
-              amfinit(&amf, 3);
-              amfstring(&amf, "nick");
-              amfnum(&amf, 0);
-              amfnull(&amf);
-              amfstring(&amf, nickname);
-              amfsend(&amf, sock);
-            }
+            // Set the given nickname
+            amfinit(&amf, 3);
+            amfstring(&amf, "nick");
+            amfnum(&amf, 0);
+            amfnull(&amf);
+            amfstring(&amf, nickname);
+            amfsend(&amf, sock);
+            // Keep what the server gave us, just in case
+            free(nickname);
+            nickname=strdup(nick);
           }
         }
       }
@@ -953,7 +972,7 @@ int main(int argc, char** argv)
       close(sock);
       return 1; // Getting banned is a failure, right?
     }
-    // from_owner: notices
+    // from_owner: notices, mute, push2talk, closing cams
     else if(amfin->itemcount==3 && amfin->items[0].type==AMF_STRING && amf_comparestrings_c(&amfin->items[0].string, "from_owner") && amfin->items[2].type==AMF_STRING)
     {
       if(!strncmp("notice", amfin->items[2].string.string, 6))
@@ -969,16 +988,22 @@ int main(int argc, char** argv)
         printf("%s %s\n", timestamp(), notice);
         fflush(stdout);
       }
-      else if(!strncmp("mute", amfin->items[2].string.string, 4))
+      else if(!strcmp("mute", amfin->items[2].string.string))
       {
         printf("%s Non-moderators have been temporarily muted.\n", timestamp());
         fflush(stdout);
       }
-      else if(!strncmp("push2talk", amfin->items[2].string.string, 9))
+      else if(!strcmp("push2talk", amfin->items[2].string.string))
       {
         printf("%s Push to talk request has been sent to non-moderators.\n", timestamp());
         fflush(stdout);
       }
+      else if(!strncmp("_close", amfin->items[2].string.string, 6) && !strcmp(&amfin->items[2].string.string[6], nickname))
+      {
+        printf("Outgoing media stream was closed\n");
+        fflush(stdout);
+        stream_stopvideo(sock, idlist_get(nickname));
+      }
     }
     // nickinuse, the nick we wanted to change to is already taken
     else if(amfin->itemcount>0 && amfin->items[0].type==AMF_STRING &&  amf_comparestrings_c(&amfin->items[0].string, "nickinuse"))
