@@ -1,6 +1,6 @@
 /*
     tc_client, a simple non-flash client for tinychat(.com)
-    Copyright (C) 2015  alicia@ion.nu
+    Copyright (C) 2015-2016  alicia@ion.nu
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as published by
@@ -31,5 +31,5 @@ extern void streamout_start(unsigned int id, int sock); // called upon privmsg "
 extern void stream_play(struct amf* amf, int sock); // called upon _result
 extern void stream_handledata(struct rtmp* rtmp);
 extern void stream_handlestatus(struct amf* amf, int sock);
-extern void stream_sendvideo(int sock, void* buf, size_t len);
+extern void stream_sendframe(int sock, void* buf, size_t len, unsigned char type);
 extern void stream_stopvideo(int sock, unsigned int id);
