@@ -27,6 +27,7 @@
   #include <sys/wait.h>
 #endif
 #include <ctype.h>
+#include <sys/stat.h>
 #include <libavcodec/avcodec.h>
 #include <libswscale/swscale.h>
 #if LIBAVUTIL_VERSION_MAJOR>50 || (LIBAVUTIL_VERSION_MAJOR==50 && LIBAVUTIL_VERSION_MINOR>37)
@@ -54,22 +55,15 @@
 #include "logging.h"
 #include "../stringutils.h"
 #include "../compat.h"
+#include "../compat_av.h"
 #include "inputhistory.h"
+#include "main.h"
 
 struct viddata
 {
   AVCodec* vdecoder;
   AVCodec* vencoder;
-  AVCodec* adecoder;
-#ifdef HAVE_AVRESAMPLE
   int audiopipe;
-  AVAudioResampleContext* resamplectx;
-#elif defined(HAVE_SWRESAMPLE)
-  int audiopipe;
-  SwrContext* swrctx;
-#endif
-  GtkTextBuffer* buffer;
-  GtkAdjustment* scroll;
 };
 struct viddata* data;
 
@@ -84,51 +78,37 @@ char frombuild=0; // Running from the build directory
   PROCESS_INFORMATION coreprocess={.hProcess=0};
 #endif
 
-void printchat(const char* text, const char* pm)
+void printchat(const char* text, const char* color, unsigned int offset, const char* pm)
 {
-  GtkAdjustment* scroll;
-  GtkTextBuffer* buffer;
+  struct chatview* chatview;
   struct user* user;
   if(pm && (user=finduser(pm)))
   {
-    pm_open(pm, 0, data->scroll);
-    scroll=user->pm_scroll;
-    buffer=user->pm_buffer;
+    pm_open(pm, 0);
+    chatview=user->pm_chatview;
   }else{
-    scroll=data->scroll;
-    buffer=data->buffer;
+    chatview=mainchat;
   }
-  char bottom=autoscroll_before(scroll);
-  // Insert new content
-  GtkTextIter end;
-  gtk_text_buffer_get_end_iter(buffer, &end);
-  gtk_text_buffer_insert(buffer, &end, "\n", -1);
-  gtk_text_buffer_insert(buffer, &end, text, -1);
-  buffer_updatesize(buffer);
-  if(bottom){autoscroll_after(scroll);}
-}
-
-void printchat_color(const char* text, const char* color, unsigned int offset, const char* pm)
-{
-  GtkAdjustment* scroll;
-  GtkTextBuffer* buffer;
-  struct user* user;
-  if(pm && (user=finduser(pm)))
-  {
-    pm_open(pm, 0, data->scroll);
-    scroll=user->pm_scroll;
-    buffer=user->pm_buffer;
-  }else{
-    scroll=data->scroll;
-    buffer=data->buffer;
-  }
-  char bottom=autoscroll_before(scroll);
+  GtkTextBuffer* buffer=gtk_text_view_get_buffer(chatview->textview);
   // Insert new content
   GtkTextIter end;
   gtk_text_buffer_get_end_iter(buffer, &end);
   gtk_text_buffer_insert(buffer, &end, "\n", -1);
   int startnum=gtk_text_iter_get_offset(&end);
-  gtk_text_buffer_insert(buffer, &end, text, -1);
+  // Insert links and regular text separately
+  const char* linktext=text;
+  const char* link;
+  while((link=strstr(linktext, "://")))
+  {
+    while(link>linktext && link[-1]!=' '){link=&link[-1];}
+    unsigned int linklen;
+    for(linklen=0; link[linklen] && link[linklen]!=' '; ++linklen);
+    if(linklen<6){continue;}
+    gtk_text_buffer_insert(buffer, &end, linktext, link-linktext);
+    gui_insert_link(buffer, &end, link, linklen);
+    linktext=&link[linklen];
+  }
+  gtk_text_buffer_insert(buffer, &end, linktext, -1);
   GtkTextIter start;
   // Set color if there was one
   if(color)
@@ -156,12 +136,12 @@ void printchat_color(const char* text, const char* color, unsigned int offset, c
     gtk_text_buffer_apply_tag_by_name(buffer, "nickname", &start, &end);
   }
   buffer_updatesize(buffer);
-  if(bottom){autoscroll_after(scroll);}
+  chatview_autoscroll(chatview);
 }
 
 unsigned int cameventsource=0;
 char buf[1024];
-gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer datap)
+gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
 {
   gsize r;
   unsigned int i;
@@ -201,8 +181,9 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
     pkt.size=size;
     int gotframe;
-    avcodec_decode_video2(cam->vctx, cam->frame, &gotframe, &pkt);
-    if(!gotframe){return 1;}
+    avcodec_send_packet(cam->vctx, &pkt);
+    gotframe=avcodec_receive_frame(cam->vctx, cam->frame);
+    if(gotframe){return 1;}
 
     if(cam->placeholder) // Remove the placeholder animation if it has it
     {
@@ -217,7 +198,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, camsize_scale.width, camsize_scale.height, AV_PIX_FMT_RGB24, SWS_BICUBIC, 0, 0, 0);
     sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
     sws_freeContext(swsctx);
-    camera_postproc(cam, cam->dstframe->data[0], camsize_scale.width, camsize_scale.height);
+    postprocess(&cam->postproc, cam->dstframe->data[0], camsize_scale.width, camsize_scale.height);
 
     GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
     GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, camsize_scale.width, camsize_scale.height, cam->dstframe->linesize[0], freebuffer, 0);
@@ -246,25 +227,30 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
       g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
       pos+=r;
     }
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
+#ifdef HAVE_LIBAO
     // Find the camera representation for the given ID (for decoder context)
     struct camera* cam=camera_find(&buf[7]);
     if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
+    if(!cam->actx && camera_init_audio(cam, frameinfo)){return 1;}
     int gotframe;
-    avcodec_decode_audio4(cam->actx, cam->frame, &gotframe, &pkt);
-    if(!gotframe){return 1;}
+    avcodec_send_packet(cam->actx, &pkt);
+    gotframe=avcodec_receive_frame(cam->actx, cam->frame);
+    if(gotframe){return 1;}
+    unsigned int samplecount=cam->frame->nb_samples*SAMPLERATE_OUT/cam->samplerate;
+    int16_t outbuf[samplecount];
+    void* outdata[]={outbuf, 0};
   #ifdef HAVE_AVRESAMPLE
-    int outlen=avresample_convert(data->resamplectx, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples);
+    int outlen=avresample_convert(cam->resamplectx, (void*)outdata, samplecount*sizeof(uint8_t), samplecount, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples);
   #else
-    int outlen=swr_convert(data->swrctx, cam->frame->data, cam->frame->nb_samples, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
+    int outlen=swr_convert(cam->swrctx, (void*)outdata, samplecount, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
   #endif
-    if(outlen>0){camera_playsnd(data->audiopipe, cam, (short*)cam->frame->data[0], outlen);}
+    if(outlen>0){camera_playsnd(data->audiopipe, cam, outbuf, outlen);}
 #endif
     return 1;
   }
   if(!strncmp(buf, "Currently online: ", 18))
   {
-    printchat(buf, 0);
+    printchat(buf, 0, 0, 0);
     char* next=&buf[16];
     while(next)
     {
@@ -295,7 +281,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   // Start streams once we're properly connected
   if(!strncmp(buf, "Currently on cam: ", 18))
   {
-    printchat(buf, 0);
+    printchat(buf, 0, 0, 0);
     if(!config_get_bool("autoopencams") && config_get_set("autoopencams")){return 1;}
     char* next=&buf[16];
     while(next)
@@ -316,7 +302,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   }
   if(buf[0]=='/') // For the /help text
   {
-    printchat(buf, 0);
+    printchat(buf, 0, 0, 0);
     return 1;
   }
   char* color=0;
@@ -424,7 +410,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     }
     if(config_get_bool("enable_logging")){logger_write(buf, channel, pm);}
     // Insert new content
-    printchat_color(buf, color, 8, pm);
+    printchat(buf, color, 8, pm);
     pm_highlight(pm);
     free(pm);
     if(space[-1]!=':') // Not a message
@@ -433,7 +419,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
       {
         if(config_get_bool("camdownonjoin"))
         {
-          gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_camera")), 0);
+          stopbroadcasting(0, 0);
         }
         space[0]=0;
         adduser(nick);
@@ -442,7 +428,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
       {
         space[0]=0;
         removeuser(nick);
-        camera_removebynick(nick);
+        camera_remove(nick, 1);
       }
       else if(!strncmp(space, " changed nickname to ", 21))
       {
@@ -468,14 +454,14 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   }
   if(!strcmp(buf, "Changed color") || !strncmp(buf, "Current color: ", 15))
   {
-    printchat_color(buf, color, 0, 0);
+    printchat(buf, color, 0, 0);
     free((void*)mycolor);
     mycolor=color;
     return 1;
   }
   if(!strncmp(buf, "Color ", 6))
   {
-    printchat_color(buf, color, 0, 0);
+    printchat(buf, color, 0, 0);
   }
   free(color);
   if(space && !strcmp(space, " is a moderator."))
@@ -499,7 +485,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   // Start a stream when someone cams up
   if(space && !strcmp(space, " cammed up"))
   {
-    printchat(buf, 0);
+    printchat(buf, 0, 0, 0);
     if(config_get_bool("autoopencams") || !config_get_set("autoopencams"))
     {
       space[0]=0;
@@ -517,16 +503,10 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     char* idend=strchr(id, ')');
     if(!idend){return 1;}
     idend[0]=0;
-    camera_removebynick(nick); // Remove any duplicates
+    camera_remove(nick, 1); // Remove any duplicates
     struct camera* cam=camera_new(nick, id);
-    cam->placeholder=g_timeout_add(100, camplaceholder_update, cam->id);
     cam->vctx=avcodec_alloc_context3(data->vdecoder);
     avcodec_open2(cam->vctx, data->vdecoder, 0);
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-    cam->actx=avcodec_alloc_context3(data->adecoder);
-    avcodec_open2(cam->actx, data->adecoder, 0);
-    cam->samples=0;
-#endif
     updatescaling(0, 0, 1);
     gtk_widget_show_all(cam->box);
     return 1;
@@ -553,23 +533,27 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   }
   if(!strncmp(buf, "VideoEnd: ", 10))
   {
-    camera_remove(&buf[10]);
+    camera_remove(&buf[10], 0);
+    return 1;
+  }
+  if(!strcmp(buf, "Outgoing media stream was closed"))
+  {
+    stopbroadcasting(0, 0);
     return 1;
   }
   if(!strncmp(buf, "Room topic: ", 12) ||
      (space && (!strcmp(space, " is not logged in") || !strncmp(space, " is logged in as ", 17))))
   {
-    printchat(buf, 0);
+    printchat(buf, 0, 0, 0);
     return 1;
   }
   if(!strcmp(buf, "Server disconnected"))
   {
-    printchat(buf, 0);
-    if(camproc)
+    printchat(buf, 0, 0, 0);
+    if(camout_cam)
     {
-      g_source_remove(cameventsource);
-      kill(camproc, SIGINT);
-      camproc=0;
+      cam_close(camout_cam);
+      camout_cam=0;
     }
     return 1;
   }
@@ -578,17 +562,18 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   return 1;
 }
 
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-void audiothread(int fd)
+#ifdef HAVE_LIBAO
+void* audiothread(void* fdp)
 {
+  int fd=*(int*)fdp;
   ao_initialize();
   ao_sample_format samplefmt;
   samplefmt.bits=16;
-  samplefmt.rate=22050;
+  samplefmt.rate=SAMPLERATE_OUT;
   samplefmt.channels=1;
   samplefmt.byte_format=AO_FMT_NATIVE; // I'm guessing libavcodec decodes it to native
   samplefmt.matrix=0;
-  ao_option clientname={.key="client_name", .value="tc_client/camviewer", .next=0};
+  ao_option clientname={.key="client_name", .value="tc_client-gtk", .next=0};
   ao_device* dev=ao_open_live(ao_default_driver_id(), &samplefmt, &clientname);
   char buf[2048];
   size_t len;
@@ -597,53 +582,78 @@ void audiothread(int fd)
     ao_play(dev, buf, len);
   }
   ao_close(dev);
+  return 0;
 }
 #endif
 
-void togglecam(GtkCheckMenuItem* item, struct viddata* data)
+void togglecam(GtkCheckMenuItem* item, void* x)
 {
   if(!gtk_check_menu_item_get_active(item))
   {
-    g_source_remove(cameventsource);
-    kill(camproc, SIGINT);
-    camproc=0;
+    if(!camout_cam){return;}
+    cam_close(camout_cam);
+    camout_cam=0;
+    if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_mic")))){return;}
+    // TODO: if switching from cam+mic to just mic, send something to tell other clients to drop the last video frame and show a mic instead
     if(camera_find("out"))
     {
       dprintf(tc_client_in[1], "/camdown\n");
-      dprintf(tc_client[1], "VideoEnd: out\n"); // Close our local display
+      camera_remove("out", 0); // Close our local display
     }
     return;
   }
-  GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
-  gtk_widget_show_all(window);
-
-  // Start a cam thread for the selected cam
-  GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
-  GIOChannel* channel=camthread(gtk_combo_box_get_active_id(combo), data->vencoder, 100000);
-  cameventsource=g_io_add_watch(channel, G_IO_IN, cam_encode, 0);
+  camselect_open(startcamout, togglecam_cancel);
 }
 
-gboolean handleresize(GtkWidget* widget, GdkEventConfigure* event, struct viddata* data)
+#ifdef HAVE_PULSEAUDIO
+void togglemic(GtkCheckMenuItem* item, void* x)
 {
-  char bottom=autoscroll_before(data->scroll);
-  if(event->width!=gtk_widget_get_allocated_width(cambox))
+  static int micpipe[2];
+  static int eventsource=0;
+  if(!gtk_check_menu_item_get_active(item)) // Stop mic broadcast
   {
-    updatescaling(event->width, 0, 0);
+    gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalk")));
+    pushtotalk_enabled=0;
+    if(!eventsource){return;}
+    close(micpipe[0]);
+    close(micpipe[1]);
+    g_source_remove(eventsource);
+    eventsource=0;
+    if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_camera")))){return;}
+    if(camera_find("out"))
+    {
+      dprintf(tc_client_in[1], "/camdown\n");
+      camera_remove("out", 0); // Close our local display
+    }
+    return;
   }
-#ifndef _WIN32 // For some reason scrolling as a response to resizing freezes windows
-  if(bottom){autoscroll_after(data->scroll);}
-#endif
-  return 0;
+  // Choose between push-to-talk and open mic
+  int choice=gtk_dialog_run(GTK_DIALOG(gtk_builder_get_object(gui, "pushtotalkdialog")));
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalkdialog")));
+  if(choice==GTK_RESPONSE_DELETE_EVENT)
+  {
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_mic")), 0);
+    return;
+  }
+  if(choice) // 1=push-to-talk, 0=open mic
+  {
+    gtk_widget_show(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalk")));
+    pushtotalk_enabled=1;
+    pushtotalk_pushed=0;
+  }
+  // Start mic broadcast
+  if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_camera"))))
+  { // Only /camup if we're not already broadcasting video
+    dprintf(tc_client_in[1], "/camup\n");
+  }
+  pipe(micpipe);
+  g_thread_new("audio_in", audiothread_in, &micpipe[1]);
+  // Attach micpipe[0] to mic_encode()
+  GIOChannel* channel=g_io_channel_unix_new(micpipe[0]);
+  g_io_channel_set_encoding(channel, 0, 0);
+  eventsource=g_io_add_watch(channel, G_IO_IN, mic_encode, 0);
 }
-
-void handleresizepane(GObject* obj, GParamSpec* spec, struct viddata* data)
-{
-  char bottom=autoscroll_before(data->scroll);
-  updatescaling(0, gtk_paned_get_position(GTK_PANED(obj)), 0);
-#ifndef _WIN32
-  if(bottom){autoscroll_after(data->scroll);}
 #endif
-}
 
 gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
 {
@@ -698,6 +708,29 @@ gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
     }
     return 1;
   }
+  if(event->keyval==GDK_KEY_Page_Up || event->keyval==GDK_KEY_KP_Page_Up ||
+     event->keyval==GDK_KEY_Page_Down || event->keyval==GDK_KEY_KP_Page_Down)
+  {
+    // Get the relevant chatview
+    GtkNotebook* tabs=GTK_NOTEBOOK(gtk_builder_get_object(gui, "tabs"));
+    int num=gtk_notebook_get_current_page(tabs);
+    struct user* user=0;
+    if(num>0)
+    {
+      GtkWidget* page=gtk_notebook_get_nth_page(tabs, num);
+      user=user_find_by_tab(page);
+    }
+    struct chatview* chatview;
+    if(user && user->pm_chatview)
+    {
+      chatview=user->pm_chatview;
+    }else{
+      chatview=mainchat;
+    }
+    // By just passing along the event we also get smooth scrolling, which GTK+ does not expose an API for
+    gtk_widget_event(GTK_WIDGET(chatview->textview), (GdkEvent*)event);
+    return 1;
+  }
   return 0;
 }
 
@@ -712,7 +745,7 @@ void sendmessage(GtkEntry* entry, void* x)
   char* pm=0;
   if(!strncmp(msg, "/pm ", 4))
   {
-    pm_open(&msg[4], 1, data->scroll);
+    pm_open(&msg[4], 1);
     gtk_entry_set_text(entry, "");
     sendingmsg=0;
     return;
@@ -772,7 +805,7 @@ void sendmessage(GtkEntry* entry, void* x)
       if(!user)
       {
         gtk_entry_set_text(entry, "");
-        printchat("No such user", 0);
+        printchat("No such user", 0, 0, 0);
         free(pm);
         sendingmsg=0;
         return;
@@ -790,7 +823,7 @@ void sendmessage(GtkEntry* entry, void* x)
     sprintf(&text[8], "%s: %s", nickname, msg);
   }
   if(config_get_bool("enable_logging")){logger_write(text, channel, pm);}
-  printchat_color(text, mycolor, 8, pm);
+  printchat(text, mycolor, 8, pm);
   gtk_entry_set_text(entry, "");
   free(pm);
   sendingmsg=0;
@@ -823,18 +856,22 @@ void startsession(GtkButton* button, void* x)
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "channelconfig")));
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordwindow")));
   gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalk")));
   const char* nick=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_nick")));
   channel=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_channel")));
   const char* chanpass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "channelpassword")));
   if(!chanpass[0]){chanpass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_password")));}
   const char* acc_user=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")));
   const char* acc_pass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")));
+  const char* cookiename=(acc_user[0]?acc_user:"no_account");
 #ifdef _WIN32
-  char cmd[strlen("./tc_client --hexcolors --cookies tinychat.cookie -u    0")+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
+  char cmd[strlen("./tc_client --hexcolors --cookies tinychat_.cookie -u    0")+strlen(cookiename)+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
   strcpy(cmd, "./tc_client --hexcolors ");
   if(config_get_bool("storecookies"))
   {
-    strcat(cmd, "--cookies tinychat.cookie ");
+    strcat(cmd, "--cookies tinychat_");
+    strcat(cmd, cookiename);
+    strcat(cmd, ".cookie ");
   }
   if(acc_user[0])
   {
@@ -862,8 +899,13 @@ void startsession(GtkButton* button, void* x)
     if(config_get_bool("storecookies"))
     {
       const char* home=getenv("HOME");
-      char filename[strlen(home)+strlen("/.tinychat.cookie0")];
-      sprintf(filename, "%s/.tinychat.cookie", home);
+
+      char filename[strlen(home)+strlen("/.config/tc_client-gtk.cookies/0")+strlen(cookiename)];
+      sprintf(filename, "%s/.config", home);
+      mkdir(filename, 0700);
+      sprintf(filename, "%s/.config/tc_client-gtk.cookies", home);
+      mkdir(filename, 0700);
+      sprintf(filename, "%s/.config/tc_client-gtk.cookies/%s", home, cookiename);
       if(acc_user[0])
       {
         execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", "-u", acc_user, "--cookies", filename, channel, nick, chanpass, (char*)0);
@@ -889,18 +931,18 @@ void startsession(GtkButton* button, void* x)
 void captcha_done(GtkWidget* button, void* x)
 {
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "captcha")));
-  gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
+  gtk_widget_show(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
   write(tc_client_in[1], "\n", 1);
 }
 
 int main(int argc, char** argv)
 {
   if(!strncmp(argv[0], "./", 2)){frombuild=1;}
-  struct viddata datax={0,0,0,0,0};
+  struct viddata datax={0,0,0};
   data=&datax;
   avcodec_register_all();
   data->vdecoder=avcodec_find_decoder(AV_CODEC_ID_FLV1);
-  data->adecoder=avcodec_find_decoder(AV_CODEC_ID_NELLYMOSER);
+  data->vencoder=avcodec_find_encoder(AV_CODEC_ID_FLV1);
 #ifndef _WIN32
   signal(SIGCHLD, SIG_IGN);
 #else
@@ -908,183 +950,17 @@ int main(int argc, char** argv)
 #endif
   config_load();
 
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-  #ifdef HAVE_AVRESAMPLE
-  data->resamplectx=avresample_alloc_context();
-  av_opt_set_int(data->resamplectx, "in_channel_layout", AV_CH_FRONT_CENTER, 0);
-  av_opt_set_int(data->resamplectx, "in_sample_fmt", AV_SAMPLE_FMT_FLT, 0);
-  // TODO: any way to get the sample rate from the frame/decoder? cam->frame->sample_rate seems to be 0
-  av_opt_set_int(data->resamplectx, "in_sample_rate", 11025, 0);
-  av_opt_set_int(data->resamplectx, "out_channel_layout", AV_CH_FRONT_CENTER, 0);
-  av_opt_set_int(data->resamplectx, "out_sample_fmt", AV_SAMPLE_FMT_S16, 0);
-  av_opt_set_int(data->resamplectx, "out_sample_rate", 22050, 0);
-  avresample_open(data->resamplectx);
-  #else
-  data->swrctx=swr_alloc_set_opts(0, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_S16, 22050, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_FLT, 11025, 0, 0);
-  swr_init(data->swrctx);
-  #endif
+#ifdef HAVE_LIBAO
   int audiopipe[2];
   pipe(audiopipe);
   data->audiopipe=audiopipe[1];
-  if(!fork())
-  {
-    prctl(PR_SET_PDEATHSIG, SIGHUP);
-    close(audiopipe[1]);
-    audiothread(audiopipe[0]);
-    _exit(0);
-  }
-  close(audiopipe[0]);
+  g_thread_new("audio", audiothread, audiopipe);
 #endif
 
   gtk_init(&argc, &argv);
-  if(frombuild)
-  {
-    gui=gtk_builder_new_from_file("gtkgui.glade");
-  }else{
-    gui=gtk_builder_new_from_file(PREFIX "/share/tc_client/gtkgui.glade");
-  }
-  gtk_builder_connect_signals(gui, 0);
-
-  unsigned int i;
-  GtkWidget* item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_broadcast_camera"));
-  g_signal_connect(item, "toggled", G_CALLBACK(togglecam), data);
-  data->vencoder=avcodec_find_encoder(AV_CODEC_ID_FLV1);
-  // Set up cam selection and preview
-  campreview.cam=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_preview"));
+  gui_init(frombuild);
   campreview.frame=av_frame_alloc();
   campreview.frame->data[0]=0;
-  GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
-  g_signal_connect(combo, "changed", G_CALLBACK(camselect_change), data->vencoder);
-  // Signals for cancelling
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
-  g_signal_connect(item, "delete-event", G_CALLBACK(camselect_cancel), 0);
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_cancel"));
-  g_signal_connect(item, "clicked", G_CALLBACK(camselect_cancel), 0);
-  // Signals for switching from preview to streaming
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_ok"));
-  g_signal_connect(item, "clicked", G_CALLBACK(camselect_accept), data->vencoder);
-  // Enable the "img" camera
-  cam_img_filepicker=camselect_file;
-  // Populate list of cams
-  unsigned int count;
-  char** cams=cam_list(&count);
-  GtkListStore* list=gtk_list_store_new(1, G_TYPE_STRING);
-  for(i=0; i<count; ++i)
-  {
-    gtk_list_store_insert_with_values(list, 0, -1, 0, cams[i], -1);
-    free(cams[i]);
-  }
-  free(cams);
-  gtk_combo_box_set_model(combo, GTK_TREE_MODEL(list));
-  g_object_unref(list);
-  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), gtk_cell_renderer_text_new(), 1);
-
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_options_settings"));
-  g_signal_connect(item, "activate", G_CALLBACK(showsettings), gui);
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_options_settings2"));
-  g_signal_connect(item, "activate", G_CALLBACK(showsettings), gui);
-
-  cambox=GTK_WIDGET(gtk_builder_get_object(gui, "cambox"));
-  userlistwidget=GTK_WIDGET(gtk_builder_get_object(gui, "userlistbox"));
-  GtkWidget* chatview=GTK_WIDGET(gtk_builder_get_object(gui, "chatview"));
-  data->scroll=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(gtk_builder_get_object(gui, "chatscroll")));
-
-  data->buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(chatview));
-  buffer_setup_colors(data->buffer);
-
-  GtkWidget* panes=GTK_WIDGET(gtk_builder_get_object(gui, "vpaned"));
-  g_signal_connect(panes, "notify::position", G_CALLBACK(handleresizepane), data);
-  gtk_paned_set_wide_handle(GTK_PANED(panes), 1);
-  panes=GTK_WIDGET(gtk_builder_get_object(gui, "chatnick"));
-  gtk_paned_set_wide_handle(GTK_PANED(panes), 1);
-
-  GtkWidget* inputfield=GTK_WIDGET(gtk_builder_get_object(gui, "inputfield"));
-  g_signal_connect(inputfield, "activate", G_CALLBACK(sendmessage), data);
-  g_signal_connect(inputfield, "key-press-event", G_CALLBACK(inputkeys), data);
-
-  // Sound
-  GtkWidget* option=GTK_WIDGET(gtk_builder_get_object(gui, "soundradio_cmd"));
-  g_signal_connect(option, "toggled", G_CALLBACK(toggle_soundcmd), gui);
-  // Logging
-  option=GTK_WIDGET(gtk_builder_get_object(gui, "enable_logging"));
-  g_signal_connect(option, "toggled", G_CALLBACK(toggle_logging), gui);
-  option=GTK_WIDGET(gtk_builder_get_object(gui, "save_settings"));
-  g_signal_connect(option, "clicked", G_CALLBACK(savesettings), gui);
-  // Youtube
-  option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_cmd"));
-  g_signal_connect(option, "toggled", G_CALLBACK(toggle_youtubecmd), gui);
-  // Misc
-  option=GTK_WIDGET(gtk_builder_get_object(gui, "camdownonjoin"));
-
-  GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "main"));
-  g_signal_connect(window, "configure-event", G_CALLBACK(handleresize), data);
-
-  // Start window and channel password window signals
-  GtkWidget* button=GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordbutton"));
-  g_signal_connect(button, "clicked", G_CALLBACK(startsession), (void*)-1); // &data);
-  button=GTK_WIDGET(gtk_builder_get_object(gui, "channelpassword"));
-  g_signal_connect(button, "activate", G_CALLBACK(startsession), (void*)-1); // &data);
-  GtkWidget* startwindow=GTK_WIDGET(gtk_builder_get_object(gui, "startwindow"));
-  // Connect signal for quick connect
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "start_menu_connect"));
-  struct channelopts cc_connect={-1,0};
-  g_signal_connect(item, "activate", G_CALLBACK(channeldialog), &cc_connect);
-  // Connect signal for the add option
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "start_menu_add"));
-  struct channelopts cc_add={-1,1};
-  g_signal_connect(item, "activate", G_CALLBACK(channeldialog), &cc_add);
-  // Connect signal for tab changing (to un-highlight)
-  item=GTK_WIDGET(gtk_builder_get_object(gui, "tabs"));
-  g_signal_connect(item, "switch-page", G_CALLBACK(pm_select), 0);
-  // Connect signal for captcha
-  g_signal_connect(gtk_builder_get_object(gui, "captcha_done"), "clicked", G_CALLBACK(captcha_done), 0);
-  // Connect signals for camera postprocessing
-  g_signal_connect(gtk_builder_get_object(gui, "cam_menu_colors"), "activate", G_CALLBACK(gui_show_camcolors), 0);
-  g_signal_connect(gtk_builder_get_object(gui, "camcolors_min_brightness"), "value-changed", G_CALLBACK(camcolors_adjust_min), 0);
-  g_signal_connect(gtk_builder_get_object(gui, "camcolors_max_brightness"), "value-changed", G_CALLBACK(camcolors_adjust_max), 0);
-  g_signal_connect(gtk_builder_get_object(gui, "camcolors_auto"), "toggled", G_CALLBACK(camcolors_toggle_auto), 0);
-  g_signal_connect(gtk_builder_get_object(gui, "camcolors_flip_horizontal"), "toggled", G_CALLBACK(camcolors_toggle_flip), 0);
-  g_signal_connect(gtk_builder_get_object(gui, "camcolors_flip_vertical"), "toggled", G_CALLBACK(camcolors_toggle_flip), (void*)1);
-  // Connect signal for hiding cameras
-  g_signal_connect(gtk_builder_get_object(gui, "cam_menu_hide"), "activate", G_CALLBACK(gui_hide_cam), 0);
-  // Load placeholder animation for cameras (no video data yet or mic only)
-  if(frombuild)
-  {
-    camplaceholder=gdk_pixbuf_animation_new_from_file("camplaceholder.gif", 0);
-  }else{
-    camplaceholder=gdk_pixbuf_animation_new_from_file(PREFIX "/share/tc_client/camplaceholder.gif", 0);
-  }
-  camplaceholder_iter=gdk_pixbuf_animation_get_iter(camplaceholder, 0);
-  // Populate saved channels
-  GtkWidget* startbox=GTK_WIDGET(gtk_builder_get_object(gui, "startbox"));
-  int channelcount=config_get_int("channelcount");
-  if(channelcount)
-  {
-    gtk_widget_destroy(GTK_WIDGET(gtk_builder_get_object(gui, "channel_placeholder")));
-  }
-  char buf[256];
-  for(i=0; i<channelcount; ++i)
-  {
-    GtkWidget* box=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-    #ifdef GTK_STYLE_CLASS_LINKED
-      GtkStyleContext* style=gtk_widget_get_style_context(box);
-      gtk_style_context_add_class(style, GTK_STYLE_CLASS_LINKED);
-    #endif
-    sprintf(buf, "channel%i_name", i);
-    const char* name=config_get_str(buf);
-    GtkWidget* connectbutton=gtk_button_new_with_label(name);
-    g_signal_connect(connectbutton, "clicked", G_CALLBACK(startsession), (void*)(intptr_t)i);
-    gtk_box_pack_start(GTK_BOX(box), connectbutton, 1, 1, 0);
-    GtkWidget* cfgbutton=gtk_button_new_from_icon_name("gtk-preferences", GTK_ICON_SIZE_BUTTON);
-    struct channelopts* opts=malloc(sizeof(struct channelopts));
-    opts->channel_id=i;
-    opts->save=1;
-    g_signal_connect(cfgbutton, "clicked", G_CALLBACK(channeldialog), opts);
-    gtk_box_pack_start(GTK_BOX(box), cfgbutton, 0, 0, 0);
-    gtk_box_pack_start(GTK_BOX(startbox), box, 0, 0, 2);
-  }
-  gtk_widget_show_all(startwindow);
-
   gtk_main();
  
 #ifdef _WIN32
@@ -1092,16 +968,7 @@ int main(int argc, char** argv)
   {
     TerminateProcess(coreprocess.hProcess, 0);
   }
-  if(camproc)
-  {
-    TerminateProcess(camproc, 0);
-  }
 #endif
   camera_cleanup();
-#ifdef HAVE_AVRESAMPLE
-  avresample_free(&data->resamplectx);
-#elif defined(HAVE_SWRESAMPLE)
-  swr_free(&data->swrctx);
-#endif
   return 0;
 }
