@@ -18,18 +18,19 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <libavcodec/avcodec.h>
-#ifndef _WIN32
-#include <sys/prctl.h>
-#endif
 #include <libswscale/swscale.h>
 #if LIBAVUTIL_VERSION_MAJOR>50 || (LIBAVUTIL_VERSION_MAJOR==50 && LIBAVUTIL_VERSION_MINOR>37)
   #include <libavutil/imgutils.h>
 #else
   #include <libavcore/imgutils.h>
 #endif
+#ifdef HAVE_PULSEAUDIO
+  #include <pulse/simple.h>
+#endif
 #include "../libcamera/camera.h"
 #include "compat.h"
 #include "../compat.h"
+#include "../compat_av.h"
 #include "gui.h"
 #include "media.h"
 
@@ -43,11 +44,6 @@ struct camera campreview={
 };
 struct camera* cams=0;
 unsigned int camcount=0;
-#ifdef _WIN32
-  PROCESS_INFORMATION camprocess={.hProcess=0};
-#else
-  pid_t camproc=0;
-#endif
 struct size camsize_out={.width=320, .height=240};
 struct size camsize_scale={.width=320, .height=240};
 GtkWidget* cambox;
@@ -55,6 +51,11 @@ GtkWidget** camrows=0;
 unsigned int camrowcount=0;
 GdkPixbufAnimation* camplaceholder=0;
 GdkPixbufAnimationIter* camplaceholder_iter=0;
+CAM* camout_cam=0;
+#ifdef HAVE_PULSEAUDIO
+char pushtotalk_enabled=0;
+char pushtotalk_pushed=0;
+#endif
 
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
 // Experimental mixer, not sure if it really works
@@ -87,48 +88,37 @@ void camera_playsnd(int audiopipe, struct camera* cam, short* samples, unsigned
 }
 #endif
 
-void camera_remove(const char* id)
+void camera_free(struct camera* cam)
 {
-  unsigned int i;
-  for(i=0; i<camcount; ++i)
-  {
-    if(!strcmp(cams[i].id, id))
-    {
-      if(cams[i].placeholder) // Remove the placeholder animation if it has it
-      {
-        g_source_remove(cams[i].placeholder);
-      }
-      gtk_widget_destroy(cams[i].box);
-      av_frame_free(&cams[i].frame);
-      avcodec_free_context(&cams[i].vctx);
+  if(cam->placeholder){g_source_remove(cam->placeholder);}
+  av_frame_free(&cam->frame);
+  avcodec_free_context(&cam->vctx);
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-      avcodec_free_context(&cams[i].actx);
-#endif
-      free(cams[i].id);
-      free(cams[i].nick);
-      --camcount;
-      memmove(&cams[i], &cams[i+1], (camcount-i)*sizeof(struct camera));
-      break;
-    }
+  if(cam->actx)
+  {
+    avcodec_free_context(&cam->actx);
+    #ifdef HAVE_AVRESAMPLE
+      avresample_free(&cam->resamplectx);
+    #else
+      swr_free(&cam->swrctx);
+    #endif
   }
-  updatescaling(0, 0, 1);
+#endif
+  free(cam->id);
+  free(cam->nick);
+
+  postproc_free(&cam->postproc);
 }
 
-void camera_removebynick(const char* nick)
+void camera_remove(const char* id, char isnick)
 {
   unsigned int i;
   for(i=0; i<camcount; ++i)
   {
-    if(!strcmp(cams[i].nick, nick))
+    if(!strcmp(isnick?cams[i].nick:cams[i].id, id))
     {
       gtk_widget_destroy(cams[i].box);
-      av_frame_free(&cams[i].frame);
-      avcodec_free_context(&cams[i].vctx);
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-      avcodec_free_context(&cams[i].actx);
-#endif
-      free(cams[i].id);
-      free(cams[i].nick);
+      camera_free(&cams[i]);
       --camcount;
       memmove(&cams[i], &cams[i+1], (camcount-i)*sizeof(struct camera));
       break;
@@ -163,6 +153,7 @@ struct camera* camera_new(const char* nick, const char* id)
   cams=realloc(cams, sizeof(struct camera)*camcount);
   struct camera* cam=&cams[camcount-1];
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
+  cam->actx=0;
   cam->samples=0;
   cam->samplecount=0;
 #endif
@@ -181,51 +172,97 @@ struct camera* camera_new(const char* nick, const char* id)
   gtk_box_pack_start(GTK_BOX(cam->box), eventbox, 0, 0, 0);
   gtk_box_pack_start(GTK_BOX(cam->box), cam->label, 0, 0, 0);
   g_signal_connect(eventbox, "button-release-event", G_CALLBACK(gui_show_cam_menu), cam->id);
+  cam->placeholder=g_timeout_add(100, camplaceholder_update, cam->id);
   // Initialize postprocessing values
-  cam->postproc.min_brightness=0;
-  cam->postproc.max_brightness=255;
-  cam->postproc.autoadjust=0;
-  cam->postproc.flip_horizontal=0;
-  cam->postproc.flip_vertical=0;
+  postproc_init(&cam->postproc);
   return cam;
 }
 
+#ifdef HAVE_LIBAO
+char camera_init_audio(struct camera* cam, uint8_t frameinfo)
+{
+  switch((frameinfo&0xc)/0x4)
+  {
+    case 0: cam->samplerate=5500; break;
+    case 1: cam->samplerate=11025; break;
+    case 2: cam->samplerate=22050; break;
+    case 3: cam->samplerate=44100; break;
+  }
+  AVCodec* decoder=0;
+  switch(frameinfo/0x10)
+  {
+    case 0x4: cam->samplerate=16000; decoder=avcodec_find_decoder(AV_CODEC_ID_NELLYMOSER); break;
+    case 0x5: cam->samplerate=8000;
+    case 0x6: decoder=avcodec_find_decoder(AV_CODEC_ID_NELLYMOSER); break;
+    case 0xb: decoder=avcodec_find_decoder(AV_CODEC_ID_SPEEX); break;
+    default:
+      printf("Unknown audio codec ID 0x%hhx\n", frameinfo/0x10);
+      return 1;
+  }
+  cam->actx=avcodec_alloc_context3(decoder);
+  avcodec_open2(cam->actx, decoder, 0);
+  #ifdef HAVE_AVRESAMPLE
+    cam->resamplectx=avresample_alloc_context();
+    av_opt_set_int(cam->resamplectx, "in_channel_layout", AV_CH_FRONT_CENTER, 0);
+    av_opt_set_int(cam->resamplectx, "in_sample_fmt", AV_SAMPLE_FMT_FLT, 0);
+    av_opt_set_int(cam->resamplectx, "in_sample_rate", cam->samplerate, 0);
+    av_opt_set_int(cam->resamplectx, "out_channel_layout", AV_CH_FRONT_CENTER, 0);
+    av_opt_set_int(cam->resamplectx, "out_sample_fmt", AV_SAMPLE_FMT_S16, 0);
+    av_opt_set_int(cam->resamplectx, "out_sample_rate", SAMPLERATE_OUT, 0);
+    avresample_open(cam->resamplectx);
+  #else
+    cam->swrctx=swr_alloc_set_opts(0, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_S16, SAMPLERATE_OUT, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_FLT, cam->samplerate, 0, 0);
+    swr_init(cam->swrctx);
+  #endif
+  return 0;
+}
+#endif
+
 void camera_cleanup(void)
 {
   unsigned int i;
   for(i=0; i<camcount; ++i)
   {
-    av_frame_free(&cams[i].frame);
-    avcodec_free_context(&cams[i].vctx);
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-    avcodec_free_context(&cams[i].actx);
-#endif
-    free(cams[i].id);
-    free(cams[i].nick);
+    camera_free(&cams[i]);
   }
   free(cams);
 }
 
 void freebuffer(guchar* pixels, gpointer data){free(pixels);}
 
-gboolean cam_encode(GIOChannel* iochannel, GIOCondition condition, gpointer datap)
+unsigned int camout_delay;
+void startcamout(CAM* cam)
 {
+  if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_mic"))))
+  { // Only /camup if we're not already broadcasting mic
+    dprintf(tc_client_in[1], "/camup\n");
+  }
+  camout_cam=cam;
+  camout_delay=500;
+  camsize_out.width=320;
+  camsize_out.height=240;
+  cam_resolution(cam, (unsigned int*)&camsize_out.width, (unsigned int*)&camsize_out.height);
+  g_timeout_add(camout_delay, cam_encode, cam);
+}
+
+gboolean cam_encode(void* camera_)
+{
+  CAM* camera=camera_;
+  if(camera!=camout_cam){return G_SOURCE_REMOVE;}
   struct camera* cam=camera_find("out");
-  char preview=0;
-  if(!cam)
+  if(cam->placeholder) // Remove the placeholder animation if it has it
   {
-    cam=&campreview;
-    preview=1;
-  }else{
-    cam->vctx->width=camsize_out.width;
-    cam->vctx->height=camsize_out.height;
-    if(!cam->dstframe->data[0])
-    {
-      cam->dstframe->format=AV_PIX_FMT_YUV420P;
-      cam->dstframe->width=camsize_out.width;
-      cam->dstframe->height=camsize_out.height;
-      av_image_alloc(cam->dstframe->data, cam->dstframe->linesize, camsize_out.width, camsize_out.height, cam->dstframe->format, 1);
-    }
+    g_source_remove(cam->placeholder);
+    cam->placeholder=0;
+  }
+  cam->vctx->width=camsize_out.width;
+  cam->vctx->height=camsize_out.height;
+  if(!cam->dstframe->data[0])
+  {
+    cam->dstframe->format=AV_PIX_FMT_YUV420P;
+    cam->dstframe->width=camsize_out.width;
+    cam->dstframe->height=camsize_out.height;
+    av_image_alloc(cam->dstframe->data, cam->dstframe->linesize, camsize_out.width, camsize_out.height, cam->dstframe->format, 1);
   }
   if(cam->frame->width!=camsize_out.width || cam->frame->height!=camsize_out.height)
   {
@@ -236,28 +273,15 @@ gboolean cam_encode(GIOChannel* iochannel, GIOCondition condition, gpointer data
     av_freep(cam->frame->data);
     av_image_alloc(cam->frame->data, cam->frame->linesize, camsize_out.width, camsize_out.height, cam->frame->format, 1);
   }
-  g_io_channel_read_chars(iochannel, (void*)cam->frame->data[0], camsize_out.width*camsize_out.height*3, 0, 0);
-  camera_postproc(cam, cam->frame->data[0], cam->frame->width, cam->frame->height);
+  cam_getframe(camera, cam->frame->data[0]);
+  postprocess(&cam->postproc, cam->frame->data[0], cam->frame->width, cam->frame->height);
   // Update our local display
   GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
   GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->frame->data[0], GDK_COLORSPACE_RGB, 0, 8, cam->frame->width, cam->frame->height, cam->frame->linesize[0], 0, 0);
-  if(!preview) // Scale, unless we're previewing
-  {
-    gdkframe=gdk_pixbuf_scale_simple(gdkframe, camsize_scale.width, camsize_scale.height, GDK_INTERP_BILINEAR);
-  }else if(gdk_pixbuf_get_height(gdkframe)>PREVIEW_MAX_HEIGHT || gdk_pixbuf_get_width(gdkframe)>PREVIEW_MAX_WIDTH) // Scale anyway if the input is huge
-  {
-    unsigned int width=gdk_pixbuf_get_width(gdkframe);
-    unsigned int height=gdk_pixbuf_get_height(gdkframe);
-    if(height*PREVIEW_MAX_WIDTH/width>PREVIEW_MAX_HEIGHT)
-    {
-      gdkframe=gdk_pixbuf_scale_simple(gdkframe, width*PREVIEW_MAX_HEIGHT/height, PREVIEW_MAX_HEIGHT, GDK_INTERP_BILINEAR);
-    }else{
-      gdkframe=gdk_pixbuf_scale_simple(gdkframe, PREVIEW_MAX_WIDTH, height*PREVIEW_MAX_WIDTH/width, GDK_INTERP_BILINEAR);
-    }
-  }
+  // Scale to fit
+  gdkframe=gdk_pixbuf_scale_simple(gdkframe, camsize_scale.width, camsize_scale.height, GDK_INTERP_BILINEAR);
   gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
   g_object_unref(oldpixbuf);
-  if(preview){return 1;}
   // Encode
   struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, cam->frame->width, cam->frame->height, AV_PIX_FMT_YUV420P, SWS_FAST_BILINEAR, 0, 0, 0);
   sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
@@ -273,107 +297,101 @@ gboolean cam_encode(GIOChannel* iochannel, GIOCondition condition, gpointer data
     .pts=AV_NOPTS_VALUE
   };
   av_init_packet(&packet);
-  avcodec_encode_video2(cam->vctx, &packet, cam->dstframe, &gotpacket);
+  avcodec_send_frame(cam->vctx, cam->dstframe);
+  gotpacket=avcodec_receive_packet(cam->vctx, &packet);
+  if(gotpacket){return G_SOURCE_CONTINUE;}
   unsigned char frameinfo=0x22; // Note: differentiating between keyframes and non-keyframes seems to break stuff, so let's just go with all being interframes (1=keyframe, 2=interframe, 3=disposable interframe)
   // Send video
   dprintf(tc_client_in[1], "/video %i\n", packet.size+1);
   write(tc_client_in[1], &frameinfo, 1);
   ssize_t w=write(tc_client_in[1], packet.data, packet.size);
-if(w!=packet.size){printf("Error: wrote %li of %i bytes\n", w, packet.size);}
+if(w!=packet.size){printf("Error: wrote %zi of %i bytes\n", w, packet.size);}
 
   av_packet_unref(&packet);
-  return 1;
+  if(camout_delay>100) // Slowly speed up to 10fps, otherwise the flash client won't show it.
+  {
+    camout_delay-=50;
+    g_timeout_add(camout_delay, cam_encode, camera_);
+    return G_SOURCE_REMOVE;
+  }
+  return G_SOURCE_CONTINUE;
 }
 
-unsigned int* camthread_delay;
-void camthread_resetdelay(int x)
+struct
 {
-  *camthread_delay=500000;
-}
-extern unsigned int cameventsource;
-GIOChannel* camthread(const char* name, AVCodec* vencoder, unsigned int delay)
+  void(*callback)(CAM* cam);
+  void(*cancelcallback)(void);
+  unsigned int eventsource;
+  CAM* current;
+  struct size size;
+} camselect={.eventsource=0, .current=0};
+
+gboolean camselect_frame(void* x)
 {
-  if(camproc)
-  {
-    g_source_remove(cameventsource);
-    kill(camproc, SIGINT);
-    usleep(200000); // Give the previous process some time to shut down
-  }
-  // Set up a pipe to be handled by cam_encode()
-  int campipe[2];
-#ifndef _WIN32
-  CAM* cam=cam_open(name); // Opening here in case of GUI callbacks
-  if(cam){cam_resolution(cam, (unsigned int*)&camsize_out.width, (unsigned int*)&camsize_out.height);}
-  pipe(campipe);
-  camproc=fork();
-  if(!camproc)
-  {
-    close(campipe[0]);
-    unsigned char img[camsize_out.width*camsize_out.height*3];
-    if(!cam) // If it failed to open, just give some grey before quitting
-    {
-      memset(img, 0x7f, camsize_out.width*camsize_out.height*3);
-      write(campipe[1], img, camsize_out.width*camsize_out.height*3);
-      _exit(1);
-    }
-#ifndef _WIN32
-    prctl(PR_SET_PDEATHSIG, SIGHUP);
-    camthread_delay=&delay;
-    signal(SIGUSR1, camthread_resetdelay);
-#endif
-    while(1)
-    {
-      usleep(delay);
-      if(delay>100000){delay-=50000;}
-      cam_getframe(cam, img);
-      write(campipe[1], img, camsize_out.width*camsize_out.height*3);
-    }
-    _exit(0);
-  }
-  if(cam){cam_close(cam);} // Leave the cam to the child process
-  close(campipe[1]);
-#else
-  char cmd[snprintf(0,0, "./tc_client-gtk-camthread %s %u", name, delay)+1];
-  sprintf(cmd, "./tc_client-gtk-camthread %s %u", name, delay);
-  w32_runcmdpipes(cmd, ((int*)0), campipe, camprocess);
-#endif
-  GIOChannel* channel=g_io_channel_unix_new(campipe[0]);
-  g_io_channel_set_encoding(channel, 0, 0);
-  return channel;
+  if(!camselect.current){return G_SOURCE_CONTINUE;}
+  GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(campreview.cam));
+  GdkPixbuf* gdkframe=scaled_gdk_pixbuf_from_cam(camselect.current, camselect.size.width, camselect.size.height, PREVIEW_MAX_WIDTH, PREVIEW_MAX_HEIGHT);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(campreview.cam), gdkframe);
+  if(oldpixbuf){g_object_unref(oldpixbuf);}
+  return G_SOURCE_CONTINUE;
 }
 
-void camselect_change(GtkComboBox* combo, AVCodec* vencoder)
+void camselect_open(void(*cb)(CAM*), void(*ccb)(void))
 {
-  if(!camproc){return;} // If there isn't a camthread already, it will be started elsewhere
-  GIOChannel* channel=camthread(gtk_combo_box_get_active_id(combo), vencoder, 100000);
-  cameventsource=g_io_add_watch(channel, G_IO_IN, cam_encode, 0);
+  camselect.callback=cb;
+  camselect.cancelcallback=ccb;
+  // Start the preview
+  if(!camselect.eventsource)
+  {
+    camselect.eventsource=g_timeout_add(100, camselect_frame, 0);
+  }
+  // Open the currently selected camera (usually starting with the top alternative)
+  if(camselect.current){cam_close(camselect.current); camselect.current=0;}
+  GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
+  camselect_change(combo, 0);
+  GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
+  gtk_widget_show_all(window);
+}
+
+void camselect_change(GtkComboBox* combo, void* x)
+{
+  if(!camselect.eventsource){return;} // Haven't opened the cam selection window yet (happens at startup)
+  if(camselect.current){cam_close(camselect.current);}
+  camselect.current=cam_open(gtk_combo_box_get_active_id(combo));
+  if(!camselect.current){return;}
+  camselect.size.width=320;
+  camselect.size.height=240;
+  cam_resolution(camselect.current, (unsigned int*)&camselect.size.width, (unsigned int*)&camselect.size.height);
 }
 
 gboolean camselect_cancel(GtkWidget* widget, void* x1, void* x2)
 {
   GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
   gtk_widget_hide(window);
-  // Note: unchecking the menu item kills the cam thread for us
-  GtkCheckMenuItem* item=GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_camera"));
-  gtk_check_menu_item_set_active(item, 0);
+  g_source_remove(camselect.eventsource);
+  camselect.eventsource=0;
+  if(camselect.current)
+  {
+    cam_close(camselect.current);
+    camselect.current=0;
+  }
+  if(camselect.cancelcallback){camselect.cancelcallback();}
   return 1;
 }
 
-extern int tc_client_in[];
-void camselect_accept(GtkWidget* widget, AVCodec* vencoder)
+void camselect_accept(GtkWidget* widget, void* x)
 {
   GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
   gtk_widget_hide(window);
-#ifndef _WIN32
-  // Tell the camthread to reset the delay to 500000 as a workaround for a quirk in the flash client
-  kill(camproc, SIGUSR1);
-#else
-  // For platforms without proper signals, resort to restarting the camthread with the new delay
-  GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
-  GIOChannel* channel=camthread(gtk_combo_box_get_active_id(combo), vencoder, 500000);
-  cameventsource=g_io_add_watch(channel, G_IO_IN, cam_encode, 0);
-#endif
-  dprintf(tc_client_in[1], "/camup\n");
+  g_source_remove(camselect.eventsource);
+  camselect.eventsource=0;
+  if(camselect.callback)
+  {
+    camselect.callback(camselect.current);
+  }else{
+    cam_close(camselect.current);
+  }
+  camselect.current=0;
 }
 
 void camselect_file_preview(GtkFileChooser* dialog, gpointer data)
@@ -411,60 +429,6 @@ const char* camselect_file(void)
   return file;
 }
 
-void camera_postproc(struct camera* cam, unsigned char* buf, unsigned int width, unsigned int height)
-{
-  if(cam->postproc.min_brightness!=0 || cam->postproc.max_brightness!=255 || cam->postproc.autoadjust)
-  {
-    unsigned char min=255;
-    unsigned char max=0;
-    unsigned int count=width*height;
-    unsigned int i;
-    for(i=0; i<count*3; ++i)
-    {
-      if(cam->postproc.autoadjust)
-      {
-        if(buf[i]<min){min=buf[i];}
-        if(buf[i]>max){max=buf[i];}
-      }
-      double v=((double)buf[i]-cam->postproc.min_brightness)*255/(cam->postproc.max_brightness-cam->postproc.min_brightness);
-      if(v<0){v=0;}
-      if(v>255){v=255;}
-      buf[i]=v;
-    }
-    if(cam->postproc.autoadjust)
-    {
-      cam->postproc.min_brightness=min;
-      cam->postproc.max_brightness=max;
-    }
-  }
-  if(cam->postproc.flip_horizontal)
-  {
-    unsigned int x;
-    unsigned int y;
-    for(y=0; y<height; ++y)
-    for(x=0; x<width/2; ++x)
-    {
-      unsigned char pixel[3];
-      memcpy(pixel, &buf[(y*width+x)*3], 3);
-      memcpy(&buf[(y*width+x)*3], &buf[(y*width+(width-x-1))*3], 3);
-      memcpy(&buf[(y*width+(width-x-1))*3], pixel, 3);
-    }
-  }
-  if(cam->postproc.flip_vertical)
-  {
-    unsigned int x;
-    unsigned int y;
-    for(y=0; y<height/2; ++y)
-    for(x=0; x<width; ++x)
-    {
-      unsigned char pixel[3];
-      memcpy(pixel, &buf[(y*width+x)*3], 3);
-      memcpy(&buf[(y*width+x)*3], &buf[((height-y-1)*width+x)*3], 3);
-      memcpy(&buf[((height-y-1)*width+x)*3], pixel, 3);
-    }
-  }
-}
-
 void updatescaling(unsigned int width, unsigned int height, char changedcams)
 {
   if(!camcount){return;}
@@ -529,7 +493,7 @@ void updatescaling(unsigned int width, unsigned int height, char changedcams)
   }
   // libswscale doesn't handle unreasonably small sizes well
   if(camsize_scale.width<8){camsize_scale.width=8;}
-  if(camsize_scale.height<1){camsize_scale.height=1;}
+  if(camsize_scale.height<6){camsize_scale.height=6;}
   // Rescale current images to fit
   for(i=0; i<camcount; ++i)
   {
@@ -552,6 +516,104 @@ gboolean camplaceholder_update(void* id)
   // Scale and replace the current image on camera
   GdkPixbuf* pixbuf=gdk_pixbuf_scale_simple(frame, camsize_scale.width, camsize_scale.height, GDK_INTERP_BILINEAR);
   gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), pixbuf);
-  g_object_unref(oldpixbuf);
+  if(oldpixbuf){g_object_unref(oldpixbuf);}
   return G_SOURCE_CONTINUE;
 }
+
+GdkPixbuf* scaled_gdk_pixbuf_from_cam(CAM* cam, unsigned int width, unsigned int height, unsigned int maxwidth, unsigned int maxheight)
+{
+  void* buf=malloc(width*height*3);
+  cam_getframe(cam, buf);
+  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(buf, GDK_COLORSPACE_RGB, 0, 8, width, height, width*3, 0, freebuffer);
+  if(height>maxheight || width>maxwidth) // Scale if the input is huge
+  {
+    GdkPixbuf* oldframe=gdkframe;
+    if(height*maxwidth/width>maxheight)
+    {
+      gdkframe=gdk_pixbuf_scale_simple(gdkframe, width*maxheight/height, maxheight, GDK_INTERP_BILINEAR);
+    }else{
+      gdkframe=gdk_pixbuf_scale_simple(gdkframe, maxwidth, height*maxwidth/width, GDK_INTERP_BILINEAR);
+    }
+    g_object_unref(oldframe);
+  }
+  return gdkframe;
+}
+
+#ifdef HAVE_PULSEAUDIO
+void* audiothread_in(void* fdp)
+{
+  int fd=*(int*)fdp;
+  pa_simple* pulse;
+  pa_sample_spec pulsespec;
+  pulsespec.format=PA_SAMPLE_FLOAT32;
+  pulsespec.channels=1;
+  pulsespec.rate=44100;
+  pulse=pa_simple_new(0, "tc_client-gtk", PA_STREAM_RECORD, 0, "mic", &pulsespec, 0, 0, 0);
+  char buf[1024];
+  // Just read/listen and write to the main thread until either reading/listening or writing fails
+  while(1)
+  {
+    if(pa_simple_read(pulse, buf, 1024, 0)<0){break;}
+    if(write(fd, buf, 1024)<1024){break;}
+  }
+  pa_simple_free(pulse);
+  close(fd);
+  return 0;
+}
+
+gboolean mic_encode(GIOChannel* iochannel, GIOCondition condition, gpointer datap)
+{
+  static AVFrame* micframe=0;
+  static AVCodecContext* avctx=0;
+  if(!micframe)
+  {
+    micframe=av_frame_alloc();
+    micframe->format=AV_SAMPLE_FMT_FLT;
+    micframe->sample_rate=44100;
+    unsigned int i;
+    for(i=0; i<AV_NUM_DATA_POINTERS; ++i)
+    {
+      micframe->data[i]=0;
+      micframe->linesize[i]=0;
+    }
+    AVCodec* encoder=avcodec_find_encoder(AV_CODEC_ID_NELLYMOSER);
+    avctx=avcodec_alloc_context3(encoder);
+    avctx->sample_fmt=AV_SAMPLE_FMT_FLT;
+    avctx->sample_rate=44100;
+    avctx->channels=1;
+    avctx->time_base.num=1;
+    avctx->time_base.den=10;
+    avcodec_open2(avctx, encoder, 0);
+  }
+  // Read up to 1024 bytes (256 samples) and start encoding
+  unsigned char buf[1024];
+  gsize r;
+  int status=g_io_channel_read_chars(iochannel, (char*)buf, 1024, &r, 0);
+  if(status!=G_IO_STATUS_NORMAL){return 0;}
+  // If push-to-talk is enabled but not pushed, return as soon as we've consumed the incoming data
+  if(pushtotalk_enabled && !pushtotalk_pushed){return 1;}
+  micframe->nb_samples=r/4; // 32bit floats
+  micframe->data[0]=buf;
+  micframe->linesize[0]=r;
+  AVPacket packet={
+#ifdef AVPACKET_HAS_BUF
+    .buf=0,
+#endif
+    .data=0,
+    .size=0,
+    .dts=AV_NOPTS_VALUE,
+    .pts=AV_NOPTS_VALUE
+  };
+  av_init_packet(&packet);
+  avcodec_send_frame(avctx, micframe);
+  if(avcodec_receive_packet(avctx, &packet)){return 1;}
+  unsigned char frameinfo=0x6c; // 6=Nellymoser, 3<<2=44100 samplerate
+  // Send audio
+  dprintf(tc_client_in[1], "/audio %i\n", packet.size+1);
+  write(tc_client_in[1], &frameinfo, 1);
+  write(tc_client_in[1], packet.data, packet.size);
+
+  av_packet_unref(&packet);
+  return 1;
+}
+#endif
